// Algorithmic Fantasy in A Flat (excerpt)
// published 2023-03-02
// for Willow Wedemeyer
(
"setup.scd".loadRelative;
"synthdefs.scd".loadRelative;
)
(
//scale
~scale = Scale.dorian(\just);

// patterns
~base = Pbind(
	\scale, ~scale,							// set tonality
	\ctranspose, -4,						// set tonality
	\group, ~src,							// put everything in its right group
	\out, ~bus[\endstage],					// output bus
	\fx, ~bus[\reverb],						// set effect bus
	\sendbus, ~bus[\delay]					// level of signal going to delay bus
);

~drone = Pbind(
	\instrument, \saw,
	\octave, 3,
	\degree, Pxrand((0..7), inf).clump(2),
	\legato, Pwhite(0.95, 1.5),
	\dur, 16,
	\amp, -16.dbamp,
	\atk, 1.5,
	\rel, 2,
	\sweep, Pwhite(1/20, 8/60),
	\rq, Pwhite(0.2, 0.6),
	\sweepmin, Pwhite(440, 660),
	\sweepmax, Pwhite(6400, 8000),
	\sweepphase, Pwhite(0, pi),
	\pan, Pwhite(-0.9, 0.9),
	\dry, 0.333,
	\send, 0
) <> ~base;

~plucks = Pbind(
	\instrument, \saw,
	\suslev, Pwhite(0.05, 0.4),
	\atk, Pwhite(0.001, 0.1),
	\dec, Pwhite(0.05, 0.4),
	\rel, Pwhite(0.4, 1.8),
	\dry, Pwhite(0.5, 0.7),
	\amp, Pseq([-9, Pn(Pwhite(-18, -12), Pwhite(3, 7))].dbamp, inf),
	\pan, Pseq([
		Pseries(-0.5, 0.1, 10),
		Pseries(0.5, -0.1, 10)
	], inf),
	\rq, Pwhite(0.1, 0.2),
	\sweepmin, 1200,
	\sweepmax, 1800,
	\legato, Pwhite(0.4, 0.7)
) <> ~base;

// building on the plucks
~durReps = Pgeom(64, 0.75, inf).asStream;
~plucks1 = Pbind(
	\octave, Pseq([
		Pwrand([5, 6], [0.1, 0.9], 12),
		Pwrand([5, 6], [0.2, 0.8], 12),
		Pwrand([5, 6], [0.3, 0.7], 12),
		Pwrand([5, 6], [0.4, 0.6], 12),
		Pwrand([5, 6], [0.5, 0.5], 12),
		Pwrand([5, 6], [0.6, 0.4], 12),
		Pwrand([5, 6], [0.7, 0.3], 12),
		Pwrand([5, 6], [0.8, 0.2], 12),
		Pwrand([5, 6], [0.9, 0.1], inf),
	]),
	\degree, Prand((-2..9), inf),
	\dur, Pdup(~durReps, Pgeom(0.0625, 1.25, 16)),
	\amp, -18.dbamp * Pwhite(0.95, 1.05),
	\send, Pwhite(0.1, 0.2)
) <> ~plucks;

~plucks2 = Pbind(
	\octave, 5,
	\dur, Pwrand([
		Pseq([0.5, 0.5]),
		Pseq([0.25, 0.25, 0.25, 0.25]),
		Pseq([1]),
	], [
		0.4,
		0.2,
		0.4
	].normalizeSum,
	inf),
	\degree, Prand([
		Pshuf([0, 2, 4, 6]),
		Pshuf([-1, 1, 3, 4]),
		Pshuf([1, 3, 5, 7])
	], inf),
	\dry, Pwhite(0.2, 0.3),
	\send, 0.5
) <> ~plucks;

~rv = Pwhite(0.99, 1.01);
~squareMel = PmonoArtic(
	\square,
	\dur, 0.5,
	\octave, Prand([\, Pdup(4, Prand([5, 6]))], inf),
	\degree, Pwalk(
			(0..7),
			Prand([1, 2], inf),
			Pseq([1, -1], inf)
			),
	\dry, 0.5 * ~rv,
	\dtune, 0.15 * ~rv,
	\atk, 0.07 * ~rv,
	\dec, 0.4 * ~rv,
	\suslev, 0.6 * ~rv,
	\rel, Pmeanrand(1.2, 2.8),
	\curve, -4 * ~rv,
	\pwidth, Pmeanrand(0.4,0.6),
	\swidth, Plprand(0.001, 0.3),
	\cutoff, Pwhite(3200,4800),
	\blend, Pseq([
		Pdup(12, Pseries(0.1, 0.025, 20)),
		Pdup(12, Pseries(0.6, -0.025, 20))
	], inf),
	\amp, 0.35 * Pwhite(0.9, 0.99),
	\legato, Phprand(0.8, 1.1),
	\send, Pseq([
		Pdup(8, Pseries(0.2, 0.05, 8)),
		Pdup(8, Pseries(0.6, -0.05, 8))
	], inf)
) <> ~base;

~pluckseg = Pbind(
	\send, Pseg([0.01, 0.6, 0.25], [60, 15], [\exp, \sine], 1),
	\amp, Pseq([-16, Pn(Pwhite(-24, -18), Pwhite(3, 7))].dbamp, inf),
) <> ~plucks2;

~droneseg = Pbind(
	\amp, Pseg([-16, -18, -28].dbamp, [20, 50], [\lin, \exp])
) <> ~drone;
);

(
~piece = Routine{
	~plucks1.play(t).reset;
	32.wait;

	~droneseg.play(t).reset;
	56.wait; // 52 sec.

	~pluckseg.play(t);
	75.wait; // 163 sec.

	c = ~squareMel.play(t).reset;
	20.wait; // 183 sec.
	c.stop;
};
);

h = ~piece.play(t);
h.stop;

// kthxbye
s.quit;
