// instruments
// saw drone
(
SynthDef(\saw, {
	arg dry = 0.5, rq = 0.5;
	var sig, env, dt;
	
	env = Env.adsr(\atk.ir(0.1), \dec.ir(0.2), \suslev.ir(1), \rel.ir(2)).kr(2, \gate.kr(1));

	dt = LFNoise2.ar(0.25!5).bipolar(\dtune.kr(0.15)).midiratio; 

	sig = VarSaw.ar(
		\freq.kr(220) * dt,
		0.0001,
		\width.kr(0.001),
		0.05
	).sum;

	sig = RLPF.ar(
		sig, 
		SinOsc.ar(
			\sweep.kr(1/20),
			\sweepphase.kr(0)
		).range(
			\sweepmin.kr(800),\sweepmax.kr(2400)
		),
		rq,
		1/(rq.sqrt)
	);

	sig = sig * env * \amp.kr(-6.dbamp);

	//sig = Splay.ar(sig);
	sig = Pan2.ar(sig, 0);

	Out.ar(\out.ir(0), sig * dry);
	Out.ar(\sendbus.ir(0), sig * \send.kr(0.25));
	Out.ar(\fx.ir(0), sig * (1- dry));
}).add;

// a square & saw voice
SynthDef(\square, {
	arg dry = 0.5;
	var sig1, sig2, sig, env, freq, dt;

	freq = \freq.kr(220, 0.02);
	dt = LFNoise2.kr(\dtfreq.kr(0.2!5)).bipolar(\dtune.kr(0.15)).midiratio;
	freq = freq * dt;
	
	env = Env.adsr(\atk.ir(0.1), \dec.ir(0.2), \suslev.ir(1), \rel.ir(2), curve: \curve.kr(-2)).kr(2, \gate.kr(1));

	//square
	sig1 = PulseDPW.ar(
		freq,
		\pwidth.kr(0.5),
		mul: 0.1
	).sum;

	// saw
	sig2 = VarSaw.ar(
		freq,
		0,
		\swidth.kr(0.0001),
		mul: 0.1
	).sum;
	sig2 = LPF.ar(sig2, \cutoff.kr(3200));

	sig = sig1.blend(sig2, \blend.kr(0.5, 0.05));
	sig = sig * env * \amp.kr(-6.dbamp);

	//sig = Splay.ar(sig);
	sig = Pan2.ar(sig, 0);

	Out.ar(\out.ir(0), sig * dry);
	Out.ar(\sendbus.ir(0), sig * \send.kr(0.25));
	Out.ar(\fx.ir(0), sig * (1- dry));
}).add;
)
