// Code-Beispiele für den SuperCollider-Workshop auf der GPN22
// 01.06.2024

/*==============================
Code ausführen
==============================*/

// * Zeilenweise: Umschalt+Enter
// * Markierten Code: Umschalt+Enter
"hello world".postln;
// Code-Statement mit ; abschließen

// Codeblöcke in runden Klammern
// Code blockweise ausführen: Strg+Enter bzw. Cmd+Enter
(
"hallo GPN22!".postln;
"Wie geht es Euch heute?".postln;
)

// Hallo Welt (das klingende)
s.boot; // Server booten (falls noch nicht geschehen)

// Für Bluetooth-Kopfhörer ggf. nötig
// Ausgabegeräte auflisten
ServerOptions.outDevices.do{|device| device.postln;};
// gewünschtes Gerät als Serveroption setzen
s.options.outDevice_("MME : Kopfhörer (L1 Stereo)");
// Server neu booten
s.reboot;

~hello = {SinOsc.ar(440, mul: 0.25)}.play;
~hello.release(2); // wieder ausschalten

/*============================
Klangfunktionen
============================*/
// UGens zum Experimentieren
SinOsc.ar;
LFTri.ar;
Saw.ar;
VarSaw.ar;
Pulse.ar;

// Rauschen
WhiteNoise.ar;
PinkNoise.ar;
BrownNoise.ar;

// Filter
LPF.ar(); // Tiefpass
HPF.ar(); // Hochpass
BPF.ar(); // Bandpass
BRF.ar(); // Band Reject/Notch-Filter

// Zufallsfunktion
LFNoise2.kr; // auch: LFNoise0 (step noise), LFNoise1 (lineare Interpolation)

// Parameter mit der Maus steuern
/* Das ist oft nützlich, um auszuprobieren, wie sich eine Funktion über ein Wertespektrum hinweg verhält.*/
MouseX.kr();
MouseY.kr();

// bleep bloop
(
~roboBleeps = {
	var sig, mod, freq = 220;
	mod = LFNoise0.kr(2).range(1, 16).round(1);
	sig = SinOsc.ar(
		freq: freq * mod,
		mul:0.1
	);
	Pan2.ar(sig, 0); //Pan2 platziert das Signal im Stereofeld: -1 ist ganz links, 1 ist ganz rechts
};
)

b = ~roboBleeps.play; // abspielen
b.release(2);

// Sägezahn-Beispiel
(
~saws = {
	arg amp = 0.8, cutoff = 3200, freq = 440, gate = 1;
	var sig, env, fsweep;
	fsweep = LFSaw.kr(0.5).range(120, 1200);
	env = Env.adsr(0.2, 0.3, 0.8, 1.5, curve: -4).kr(2, gate: gate);
	sig = VarSaw.ar(fsweep, 0, 0.001, 0.1);
	sig = LPF.ar(in: sig, freq: cutoff);
	sig = sig * env * amp;
	sig = Pan2.ar(sig, 0);
};
)

~p1 = ~saws.play(args: [\amp, -6.dbamp]);
~p1.set(\cutoff, 800);
~p1.set(\gate, 0);

// Hochpassfilter, kontrolliert mit MouseX
(
~mouseCtl = {
	var sig, mouse;
	mouse = MouseX.kr(50, 10000, warp: 1, lag: 0.05);
	sig = HPF.ar(
		VarSaw.ar(mouse, width: 0.5, mul: 0.25),
		50
	);
	sig = Pan2.ar(sig, 0);
};
)

e = ~mouseCtl.play;
e.release(2);

// Rauschen mit Bandpassfilter
(
~filtered_noise = {
	arg amp = 0.8;
	var sig, centerFreq, filterQuality;
	centerFreq = MouseX.kr(100, 8000, 1);
	filterQuality = MouseY.kr(0.5, 0.001, 0);
	sig = BPF.ar(
		WhiteNoise.ar,
		centerFreq,
		filterQuality,
		1/filterQuality.sqrt // Ausgleich für die reduzierte Lautstärke durch den Bandpassfilter
		);
	sig = Pan2.ar(sig * amp, 0);
};
)
~p2 = ~filtered_noise.play;
~p2.release(2);

// Werte begrenzen mit .clip
(
~filterFunktion = {
	arg cutoff = 1600;
	var sig;
	sig = LPF.ar(
		VarSaw.ar(330, width: 0.001, mul: 0.1),
		cutoff.clip(20, 20000);
	);
	sig = Pan2.ar(sig, 0);
}
)

d = ~filterFunktion.play;
d.set(\cutoff,15); // jetzt hören wir zwar nichts, es passiert aber auch nichts Schlimmes!
d.set(\cutoff, 990);
d.release(2);

(
~scaleExample = {
	var env, freq, sig;
	// wähle einen zufälligen Ton aus einer pentatonischen Skala und wähle eine zufällige Oktave
	env = Env.perc(0.01, 1.2).kr(Done.freeSelf);
	freq = (Scale.majorPentatonic.degrees.choose + [36, 48, 60, 72].choose).midicps;
	sig = SinOsc.ar(freq, mul: 0.4);
	sig = Pan2.ar(sig * env);
}.play;
)

/*=========================
Synthdefs
=========================*/
// Minimalbeispiel
(
SynthDef(\minimal, {
	arg freq = 440, amp = 0.5;
	var sig;
	sig = SinOsc.ar(freq, mul: 0.5);
	sig = sig * amp;
	sig = Pan2.ar(sig, 0);
	Out.ar(0, sig);
}).add; // .add lädt die Synthdef auf den Server
)
f = Synth(\minimal, [\freq, 550, \amp, 0.25]);
f.free;

// ein etwas ausführlicheres Beispiel
(
SynthDef(\saws, {
	arg amp = 0.8, atk = 0.01, dec = 0.2, suslev = 1, rel = 1.5, gate = 1, cutoff = 3200, freq = 440;
	var sig, env, dtune;

	dtune = [0.995, 0.997, 1, 1.002, 1.005];
	env = Env.adsr(atk, dec, suslev, rel).kr(Done.freeSelf, gate);

	sig = VarSaw.ar(freq * dtune, 0, 0.001, 0.1);
	sig = LPF.ar(in: sig, freq: cutoff);
	sig = sig * amp * env;
	sig = Splay.ar(sig);

	Out.ar(0, sig);
}).add;
)

i = Synth(\saws, [\freq, 110, \amp, 0.5]);
i.set(\freq, 60.midicps);
i.set(\cutoff, 880);
i.set(\gate, 0);

/*==========================
Patterns
==========================*/
// Minimalbeispiel: Eine Tonleiter
(
k = Pbind(
	\degree, Pseq([0, 1, 2, 3, 4, 5, 6, 7], 1),
	\dur, 0.5,
	\amp, 0.35
).play;
)

// Ein etwas ausgefeilteres Beispiel
(
~sawPat = Pbind(
	\instrument, \saws,
	\scale, Scale.minorPentatonic,
	\root, -2,
	\octave, Prand([2, 3, 4, 5, 6], inf),
	\degree, Pclump(
		Pwrand([1, 2, 3], [0.75, 0.1, 0.15], inf),
		Prand([0, 1, 2, 3, 4, 5], inf)
		),
	\dur, Pseq([Pgeom(0.125, 1.5, 8)], inf),
	\amp, Pwhite(0.45, 0.6),
	\atk, Pwhite(0.001, 1.25),
	\dec, Pwhite(0.05, 0.5),
	\suslev, Prand([1, 0.5, 0.1], inf),
	\rel, Pwhite(0.45, 2),
	\cutoff, Pseq([Pgeom(600, 1.5, 8)], inf),
	\legato, Pseq([Pseries(0.5, 0.25, 6)], inf),
	\strum, Plprand(0.05, 0.25),
	\ctranspose, -3
);

l = ~sawPat.play;
)

l.stop;

// Bonustrack: Patterns "on the fly" ändern
// mit Pdef
(
Pdef(\sawPat,
	Pbind(
		\instrument, \saws,
		\scale, Scale.minorPentatonic,
		\root, 2,
		\octave, Prand([2, 3, 4, 5, 6], inf),
		\degree, Pclump(
			Pwrand([1, 2, 3], [0.75, 0.1, 0.15], inf),
			Prand([0, 1, 2, 3, 4, 5], inf)
			),
		\dur, Pseq([Pgeom(0.125, 1.5, 12)], inf),
		\amp, Pwhite(0.45, 0.6),
		\atk, Pwhite(0.001, 1.25),
		\dec, Pwhite(0.05, 0.5),
		\suslev, Prand([1, 0.5, 0.1], inf),
		\rel, Pwhite(0.45, 2),
		\cutoff, Pseq([Pgeom(600, 1.5, 8)], inf),
		\legato, Pseq([Pseries(0.5, 0.25, 6)], inf),
		\strum, Plprand(0.05, 0.25),
		\ctranspose, 0
	 )
).play;
)
Pdef(\sawPat).stop;

// mit Pbindef
(
Pbindef(\sawPat,
	\instrument, \saws,
	\scale, Scale.minorPentatonic,
	\root, 2,
	\octave, Prand([2, 3, 4, 5, 6], inf),
	\degree, Pclump(
		Pwrand([1, 2, 3], [0.75, 0.1, 0.15], inf),
		Prand([0, 1, 2, 3, 4, 5], inf)
		),
	\dur, Pseq([Pgeom(0.125, 1.5, 12)], inf),
	\amp, Pwhite(0.45, 0.6),
	\atk, Pwhite(0.001, 1.25),
	\dec, Pwhite(0.05, 0.5),
	\suslev, Prand([1, 0.5, 0.1], inf),
	\rel, Pwhite(0.45, 2),
	\cutoff, Pseq([Pgeom(600, 1.5, 8)], inf),
	\legato, Pseq([Pseries(0.5, 0.25, 6)], inf),
	\strum, Plprand(0.05, 0.25),
	\ctranspose, 0
).play;
)
Pbindef(\sawPat, \ctranspose, 1);
Pbindef(\sawPat).stop;
